/*
 * Desenvolvido para a disciplina Programacao 1
 * Curso de Bacharelado em Cincia da Computao
 * Departamento de Sistemas e Computao
 * Universidade Federal da Paraba
 *
 * Copyright (C) 1999 Universidade Federal da Paraba.
 * No redistribuir sem permisso.
 */
package p1.aplic.correio;

import p1.io.*;
import p1.util.*;
import java.util.*;

/**
 * Classe que implementa uma interface simples (a caractere)
 * de manipulao de mensagens de correio eletrnico.
 * <p>
 * O programa manipula uma caixa postal de mensagens de correio eletrnico.
 * O funcionamento da interface  como segue. Um objeto dessa
 * classe deve ser criado com um argumento especificando o titular da caixa postal
 * a ser manipulada. Ao chamar o mtodo principal (interfaceComUsurio()),
 * o contedo da caixa postal pode ser manipulado e novas mensagens podem ser enviadas.
 * Pode-se enviar correio textual, de "misso impossvel" ou de udio.
 *
 * @author   Jacques Philippe Sauv, jacques@dsc.ufpb.br
 * @version 1.1
 * <br>
 * Copyright (C) 1999 Universidade Federal da Paraba.
 */

public class CorreioIU3 {
    private CaixaPostal caixa;
    
    /**
     * Construtor de uma interface com o usurio para manipular
     * uma caixa postal de correio eletrnico.
     * @param titular O titular da caixa postal que se deseja manipular.
     */
    public CorreioIU3(String titular) {
        caixa = new CaixaPostal(titular);
    }

    /**
     * Interface com o usurio para a manipulao de caixa postal.
     * A interface consiste basicamente de um lao de interpretao de comandos.
     * O contedo da caixa postal  mostrado na tela e um menu de comandos
     *  exibido. Os comandos disponveis so:
     * <p><strong>exibir</strong>: exibe a mensagem corrente.
     * <p><strong>texto</strong>: envia uma nova mensagem textual de correio.
     * Neste caso, deve-se especificar ainda o destinatrio, o assunto e
     * o contedo da mensagem propriamente dita. O remetente ser o titular da caixa postal.
     * <p><strong>missaoimpossivel</strong>: envia uma nova mensagem textual de correio.
     * Como o caso de mensagem de texto  mas a mensagem vai se auto-destruir depois de lida.
     * <p><strong>voz</strong>: envia uma nova mensagem de correio usando udio.
     * Neste caso, deve-se especificar ainda o destinatrio, o assunto e
     * o arquivo contendo o udio da mensagem. Essa interface estranha foi usada porque
     * no queremos obrigar o uso de um microfone para gravar a mensagem.
     * O remetente ser o titular da caixa postal.
     * <p><strong>excluir</strong>: marca a mensagem corrente para excluso. Chamamos isso
     * de <strong>excluso lgica</strong>. A excluso em s (fsica)
     *  feita ao salvar a caixa postal no final (vide o comando "quit").
     * <p><strong>+</strong>: avana para a prxima mensagem.
     * <p><strong>-</strong>: recua para a mensagem anterior.
     * <p><strong>quit</strong>: encerra a manipulao da caixa postal. Pede-se neste momento
     * se a caixa postal deve ser salva em disco ou no. Responda com 's' ou 'n'.
     * <p>Os comandos podem ser digitados de forma abreviada, desde que um nmero
     * suficiente de letras seja informado. Por exemplo, basta digitar "v" para
     * enviar uma mensagem nova de udio.
     * Digitar "ex" no  suficiente, pois h dois comandos comeando
     * com "ex". Deve-se digitar pelo menos "exc" (excluir) ou "exi" (exibir).
     */
    public void interfaceComUsurio() {
        while(true) {
            mostraResumoCaixaPostal(caixa);
            String cmd = Entrada.in.lerLinha("exibir, texto, missaoimpossivel, voz, excluir, +, -, quit? ");
            if(cmd.startsWith("exi")) {
                Mensagem m = caixa.mensagemCorrente();
                if(m != null) {
                    m.exibir();
                }
            } else if(cmd.startsWith("t")) {
                String destinatrio = obtemUmaLinha("Para quem? ");
                String assunto = obtemUmaLinha("Assunto? ");
                String contedo = obtemVriasLinhas("Conteudo da mensagem? (. para terminar) ");
                enviarMensagemTexto(caixa.getTitular(), destinatrio, assunto, contedo);
            } else if(cmd.startsWith("m")) {
                String destinatrio = obtemUmaLinha("Para quem? ");
                String assunto = obtemUmaLinha("Assunto? ");
                String contedo = obtemVriasLinhas("Conteudo da mensagem? (. para terminar) ");
                enviarMensagemMissaoImpossivel(caixa.getTitular(), destinatrio, assunto, contedo);
            } else if(cmd.startsWith("v")) {
                String destinatrio = obtemUmaLinha("Para quem? ");
                String assunto = obtemUmaLinha("Assunto? ");
                String clip = obtemUmaLinha("Arquivo de clip de audio? ");
                enviarMensagemudio(caixa.getTitular(), destinatrio, assunto, clip);
            } else if(cmd.startsWith("exc")) {
                caixa.excluir();
            } else if(cmd.startsWith("+")) {
                caixa.avanar();
            } else if(cmd.startsWith("-")) {
                caixa.recuar();
            } else if(cmd.startsWith("q")) {
                cmd = Entrada.in.lerLinha("Salvar CaixaPostal? ");
                if(cmd.startsWith("s")) {
                    caixa.salvar();
                }
                break;
            } else {
                System.out.println("Comando <" + cmd + "> desconhecido");
            }
        }
    }

    /**
     * Exibe na sada padro um resumo da mensagens presentes na caixa postal.
     * @param caixa A caixa postal a ser exibida
     */
    // Observe que este mtodo *no*  public
    private void mostraResumoCaixaPostal(CaixaPostal caixa) {
        Iterator it = caixa.iterator();
        if(!it.hasNext()) {
            System.out.println("Nao ha mensagem.");
            return;
        }
        // -16.16s significa um string (s) alinhado  esquerda (-)
        // com um mnimo de 16 caracteres e um mximo de 16 caracteres
        Formata f1 = new Formata("%-16.16s");
        Formata f2 = new Formata("%-40.40s");
        Formata f3 = new Formata("%3d");
        Formata f4 = new Formata("%-12.12s");
        System.out.println("  Num " + f4.form("Remetente") +
                           " " + f1.form("Data") +
                           " " + f2.form("Assunto"));
        int numMensagem = 1;
        while(it.hasNext()) {
            Mensagem m = (Mensagem)it.next();
            String cursor = " ";
            if(m == caixa.mensagemCorrente()) {
                cursor = ">";
            }
            String estado = m.isExcluda() ? "X" : " ";
            System.out.println(cursor + estado +
                                f3.form(numMensagem) + " " +
                                f4.form(m.getRemetente()) + " " +
                                f1.form(m.getDataEnvio().DDMMAAAAHHMM()) + " " +
                                f2.form(m.getAssunto()));
            numMensagem++;
        }
    }

    /** Mtodo auxiliar para obter uma linha da entrada padro, tendo
     * o cuidado de no aceitar uma linha vazia.
     * @param prompt O prompt a exibir ao usurio antes de ler a informao.
     * @return A linha lida.
     */
    // Observe que este mtodo *no*  public
    private String obtemUmaLinha(String prompt) {
        String linha;
        while((linha = Entrada.in.lerLinha(prompt)).equals("")) {
            System.out.println("Favor fornecer alguma informacao");
        }
        return linha;
    }

    /** Mtodo auxiliar para obter vrias linhas da entrada padro.
     * O final da entrada  indicado digitando "." sozinho no incio
     * de uma linha.
     * @param prompt O prompt a exibir ao usurio antes de ler a informao.
     * @return As linhas lida, como string nico. As linhas so separadas
     * por um caractere de separao de linha apropriado.
     */
    // Observe que este mtodo *no*  public
    private String obtemVriasLinhas(String prompt) {
        String resposta = "";
        String separador = System.getProperty("line.separator");
        String linha;
        System.out.println(prompt);
        while(!(linha = Entrada.in.lerLinha("")).equals(".")) {
            resposta += linha + separador;
        }
        return resposta;
    }

    /**
     * Envia uma mensagem textual de correio eletrnico para um destinatrio.
     * @param remetente O remetente da mensagem.
     * @param destinatrio O destinatrio da mensagem.
     * @param assunto O assunto da mensagem.
     * @param contedo O contedo da mensagem, podendo conter vrias linhas de texto.
     */
    // Observe que este mtodo *no*  public
    private void enviarMensagemTexto(String remetente, String destinatrio, String assunto, String contedo) {
        CaixaPostal caixaDestino = new CaixaPostal(destinatrio);
        caixaDestino.inserir(new MensagemTexto(remetente, assunto, contedo));
        caixaDestino.salvar();
    }

    /**
     * Envia uma mensagem textual de correio eletrnico de Jim para Tom Cruise.
     * @param remetente O remetente da mensagem.
     * @param destinatrio O destinatrio da mensagem.
     * @param assunto O assunto da mensagem.
     * @param contedo O contedo da mensagem, podendo conter vrias linhas de texto.
     */
    // Observe que este mtodo *no*  public
    private void enviarMensagemMissaoImpossivel(String remetente, String destinatrio, String assunto, String contedo) {
        CaixaPostal caixaDestino = new CaixaPostal(destinatrio);
        caixaDestino.inserir(new MensagemMissaoImpossivel(remetente, assunto, contedo));
        caixaDestino.salvar();
    }
    /**
     * Envia uma mensagem de udio de correio eletrnico para um destinatrio.
     * @param remetente O remetente da mensagem.
     * @param destinatrio O destinatrio da mensagem.
     * @param assunto O assunto da mensagem.
     * @param clip O arquivo de udio contendo a mensagem.
     */
    // Observe que este mtodo *no*  public
    private void enviarMensagemudio(String remetente, String destinatrio, String assunto, String clip) {
        CaixaPostal caixaDestino = new CaixaPostal(destinatrio);
        caixaDestino.inserir(new MensagemAudio(remetente, assunto, clip));
        caixaDestino.salvar();
    }
}
