/*
 * Desenvolvido para a disciplina Programacao 1
 * Curso de Bacharelado em Cincia da Computao
 * Departamento de Sistemas e Computao
 * Universidade Federal da Paraba
 *
 * Copyright (C) 1999 Universidade Federal da Paraba.
 * No redistribuir sem permisso.
 */
package p1.aplic.correio;

import p1.aplic.geral.*;
import java.io.*;

/**
 * Classe que representa uma mensagem normal de correio eletronico.
 *
 * @author   Jacques Philippe Sauv, jacques@dsc.ufpb.br
 * @version 1.1
 * <br>
 * Copyright (C) 1999 Universidade Federal da Paraba.
 */

public class MensagemTexto extends MensagemAbstrata implements Serializable {
  static final long serialVersionUID = 7693744891877165882L;
  protected String  contedo;

  /**
   * Cria uma mensagem textual de correio eletrnico
   * @param remetente O remetente da mensagem
   * @param assunto O assunto da mensagem
   * @param contudo O contedo da mensagem, podendo conter vrias linhas
   */
  public MensagemTexto(String remetente, String assunto, String contedo) {
    super(remetente, assunto);
    this.contedo = contedo;
  }

  /**
   * Recupera o contedo da mensagem.
   * O contedo  um String podendo conter vrias linhas.
   * @return O contedo da mensagem
   */
  public String getContedo() {
    return contedo;
  }

  /**
   * Exibir a mensagem. Os dados da mensagem so apresentados na sada padro.
   * Aps este mtodo, a mensagem  considerada "lida".
   */
  public void exibir() {
    System.out.println("De: " + remetente);
    System.out.println("Data: " + dataEnvio.DDMMAAAAHHMM());
    System.out.println("Assunto: " + assunto);
    System.out.println(contedo);
    estado |= LIDA;
  }

  /**
   * Testa a igualdade de um objeto com esta mensagem.
   * @param objeto O objeto a comparar com esta mensagem.
   * @return true se o objeto for igual a esta mensagem, false caso contrrio.
   */
  public boolean equals(Object objeto) {
    if(! (objeto instanceof MensagemTexto)) {
      return false;
    }
    MensagemTexto outra = (MensagemTexto)objeto;
    return super.equals(objeto) &&
           getContedo().equals(outra.getContedo());
  }

  /**
   * Fornea uma representao da mensagem como String
   * @return A representao da mensagem como String.
   */
  public String toString() {
    return "Remetente: " + remetente +
           ", Data: " + dataEnvio.DDMMAAAAHHMM() +
           ", Assunto: " + assunto +
           ", Contedo: " + contedo;
  }
}
