/*
 * Desenvolvido para a disciplina Programacao 2
 * Curso de Bacharelado em Cincia da Computao
 * Departamento de Sistemas e Computao
 * Universidade Federal da Paraba
 *
 * Copyright (C) 2001 Universidade Federal da Paraba.
 * No redistribuir sem permisso.
 */
 
package p1.aplic.mancala.jogo;
import java.util.*;

/**
 * Um jogador automtico mais-ou-menos razovel para jogar Mancala.
 *
 * @author   Jacques Philippe Sauv, jacques@dsc.ufpb.br
 * @version 1.0
 * <br>
 * Copyright (C) 2001 Universidade Federal da Paraba.
 */
public class JogadorComputador2 extends JogadorAbstrato {
  /**
   * Construtor de um jogador automtico (computador)
   * com algoritmo razovel para jogar.
   *
   * @param   nome   O nome do jogador.
   * @param   posio   A posio do jogador (EM_BAIXO ou EM_CIMA).
   */
  public JogadorComputador2(String nome, int posio) {
    super(nome, posio);
  }

  /**
   * Pede ao jogador para escolher um buraco para jogar.
   * A estratgia  razovel: tenta todos os buracos e escolhe o que
   * d "melhor" resultado, numa avaliao simples.
   *
   * @param   jogo   O jogo do qual o jogador est participando.
   * @return  O nmero do buraco (entre 0 e 5)
   * @throws  MancalaException se o buraco escolhido no for possvel.
   */
  public int escolheJogada(JogoMancala jogo) throws MancalaException {
    // Jogador eh computador: devemos determinar a melhor jogada
    int melhorJogada = -1;
    int jogadaComRepetio = -1;
    int maxPedrasAdicionais = -1;

    // tente as possiveis jogadas
    Tabuleiro tabuleiroOriginal = jogo.getTabuleiro();
    int sementesOriginais = jogo.getKalaha(this).getNmeroDeSementes();
    for(int i = 0; i < Tabuleiro.NUM_BURACOS; i++) {
      if(tabuleiroOriginal.getBuraco(getPosio(), i).getNmeroDeSementes() > 0) {
        Tabuleiro tabuleiroDeTeste = (Tabuleiro)tabuleiroOriginal.clone();
        jogo.setTabuleiro(tabuleiroDeTeste);
        if(jogo.umaJogada(i, JogoMancala.SEM_EVENTO) == this) {
          jogadaComRepetio = i;
        }
        int pedrasAdicionais = jogo.getKalaha(this).getNmeroDeSementes() -
                               sementesOriginais;
        if( pedrasAdicionais > maxPedrasAdicionais ) {
          maxPedrasAdicionais = pedrasAdicionais;
          melhorJogada = i;
        }
      }
    }
    jogo.setTabuleiro(tabuleiroOriginal);

    // tentamos todas as possibilidades: escolhe a melhor
    if(maxPedrasAdicionais > 1) {
      return melhorJogada;
    } else if(jogadaComRepetio >= 0) {
      return jogadaComRepetio;
    } else {
      return melhorJogada;
    }
  }
}

