/**
 * Classe de conta bancria simples.
 *
 * @author   Jacques Philippe Sauv, jacques@dsc.ufpb.br
 * @version 1.1
 * <br>
 * Copyright (C) 1999 Universidade Federal da Paraba.
 */

import p1.aplic.geral.*;

public class ContaSimples2 {
	// atributos
	private int nmero;

	private Pessoa titular;

	private double saldo;

	// construtores
	/**
	 * Cria uma conta a partir de uma pessoa e nmero de conta.
	 * 
	 * @param titular
	 *            O titular da conta.
	 * @param nmero
	 *            O nmero da conta.
	 */
	public ContaSimples2(Pessoa titular, int nmero) {
		this.nmero = nmero;
		this.titular = titular;
		saldo = 0.0;
	}

	/**
	 * Cria uma conta a partir de um nome e cpf de pessoa fsica, e um nmero de
	 * conta.
	 * 
	 * @param nome
	 *            O nome do titular da conta.
	 * @param cpf
	 *            O CPF do titular da conta.
	 * @param nmero
	 *            O nmero da conta.
	 */
	// h sobrecarga do mtodo construtor
	public ContaSimples2(String nome, String cpf, int nmero) {
		// aninhamento de mtodo construtor
		this(new Pessoa(nome, cpf), nmero);
	}

	/**
	 * Recupera o nmero da conta.
	 * 
	 * @return O nmero da conta.
	 */
	public int getNmero() {
		return nmero;
	}

	/**
	 * Recupera o titular da conta.
	 * 
	 * @return O titular da conta.
	 */
	public Pessoa getTitular() {
		return titular;
	}

	/**
	 * Recupera o nome do titular da conta.
	 * 
	 * @return O nome do titular da conta.
	 */
	/* feito para ajudar os principiantes escondendo a classe Pessoa no inicio */
	public String getNome() {
		// aninhamento de mtodo
		return titular.getNome();
	}

	/**
	 * Recupera o CPF do titular da conta.
	 * 
	 * @return O CPF do titular da conta.
	 */
	/* feito para ajudar os principiantes escondendo a classe Pessoa no inicio */
	public String getCPF() {
		return titular.getCPF();
	}

	/**
	 * Recupera o saldo da conta.
	 * 
	 * @return O saldo da conta.
	 */
	public double getSaldo() {
		return saldo;
	}

	/**
	 * Efetua um depsito numa conta.
	 * 
	 * @param valor
	 *            O valor a depositar.
	 */
	public void depositar(double valor) {
		// credita a conta
		saldo += valor;
	}

	/**
	 * Efetua sacada na conta.
	 * 
	 * @param valor
	 *            O valor a sacar.
	 * @return O sucesso ou no da operao.
	 */
	public boolean sacar(double valor) {
		// debita a conta
		if (saldo - valor >= 0) {
			saldo -= valor;
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Transforma os dados da conta em um String.
	 * 
	 * @return O string com os dados da conta.
	 */
	public String toString() {
		return "numero " + nmero + ", nome " + getNome() + ", saldo " + saldo;
	}
}
