/*
 * Desenvolvido para a disciplina Programacao 1
 * Curso de Bacharelado em Cincia da Computao
 * Departamento de Sistemas e Computao
 * Universidade Federal da Paraba
 *
 * Copyright (C) 1999 Universidade Federal da Paraba.
 * No redistribuir sem permisso.
 */

import p1.aplic.geral.Data;

/**
 * Classe que representa uma mensagem normal de correio eletronico.
 *
 * Uma mensagem contm um remetente, um assunto uma data de envio e algum contedo.
 * O contedo depende do tipo exato de mensagem (textual, udio).
 * Uma mensagem pode ser exibida (lida) e marcada para excluso.
 *
 * @author   Jacques Philippe Sauv, jacques@dsc.ufpb.br
 * @version 1.1
 * <br>
 * Copyright (C) 1999 Universidade Federal da Paraba.
 */

public class Mensagem {
    private static final int LIDA = 0x1;
    private static final int EXCLUDA = 0x2;
    private static final int NOVA = ~(LIDA | EXCLUDA);

    private String  remetente;
    private String  assunto;
    private String  contedo;
    private Data    dataEnvio;
    private int     estado;

    public Mensagem(String remetente, String assunto, String contedo) {
        this.remetente = remetente;
        this.assunto = assunto;
        this.contedo = contedo;
        dataEnvio = new Data();
        estado = NOVA;
    }

    /**
     * Recupera o remetente da mensagem
     * @return O remetente da mensagem
     */
    public String getRemetente() {
        return remetente;
    }

    /**
     * Recupera o assunto da mensagem
     * @return O assunto da mensagem
     */
    public String getAssunto() {
        return assunto;
    }

    /**
     * Recupera o contedo da mensagem
     * @return O contedo da mensagem
     */
    public String getContedo() {
        return contedo;
    }

    /**
     * Recupera a data de envio da mensagem
     * @return A data de envio da mensagem
     */
    public Data getDataEnvio() {
        return dataEnvio;
    }

    /**
     * Informa se a mensagem foi lida ou no
     * @return true se a mensagem foi lida, false caso contrrio
     */
    public boolean isLida() {
        return (estado & LIDA) == LIDA;
    }

    /**
     * Informa se a mensagem foi excluda ou no
     * @return true se a mensagem foi excluda, false caso contrrio
     */
    public boolean isExcluda() {
        return (estado & EXCLUDA) == EXCLUDA;
    }

    /**
     * Marcar a mensagem como excluda.
     * A excluso deve ser feita pela coleo que armazena as mensagens.
     * Um exemplo de tal coleo  CaixaPostal.
     */
    public void excluir() {
        estado |= EXCLUDA;
    }

    /**
     * Marcar a mensagem como no excluda.
     */
    public void marcarNoExcluda() {
        estado &= ~EXCLUDA;
    }

    /**
     * Marcar a mensagem como no lida.
     */
    public void marcarNoLida() {
        estado &= ~LIDA;
    }

    /**
     * Testa a igualdade de um objeto com esta mensagem.
     * @param objeto O objeto a comparar com esta mensagem.
     * @return true se o objeto for igual a esta mensagem, false caso contrrio.
     */
    public boolean equals(Object objeto) {
        if(! (objeto instanceof Mensagem)) {
            return false;
        }
        Mensagem outra = (Mensagem)objeto;
        return getRemetente().equals(outra.getRemetente())
                && getAssunto().equals(outra.getAssunto())
                && getContedo().equals(outra.getContedo());
    }

    /**
     * Exibir a mensagem. Os dados da mensagem so apresentados na sada padro.
     * Aps este mtodo, a mensagem  considerada "lida".
     */
    public void exibir() {
        System.out.println("De: " + remetente);
        System.out.println("Data: " + dataEnvio.DDMMAAAAHHMM());
        System.out.println("Assunto: " + assunto);
        System.out.println(contedo);
        estado |= LIDA;
    }

    /**
     * Fornea uma representao da mensagem como String
     * @return A representao da mensagem como String.
     */
    public String toString() {
        return "Remetente: " + remetente +
               ", Data: " + dataEnvio.DDMMAAAAHHMM() +
               ", Assunto: " + assunto +
               ", Contedo: " + contedo;
    }
}

