/*
 * Desenvolvido para a disciplina Programacao 1
 * Curso de Bacharelado em Cincia da Computao
 * Departamento de Sistemas e Computao
 * Universidade Federal da Paraba
 *
 * Copyright (C) 1999 Universidade Federal da Paraba.
 * No redistribuir sem permisso.
 */

import java.applet.Applet;
import java.applet.AudioClip;
import java.net.URL;

import p1.aplic.geral.Data;

/**
 * Classe que representa uma mensagem de udio de correio eletronico.
 * 
 * @author Jacques Philippe Sauv, jacques@dsc.ufpb.br
 * @version 1.1 <br>
 *          Copyright (C) 1999 Universidade Federal da Paraba.
 */

public class MensagemAudio implements p1.aplic.correio.Mensagem {
	private static final int LIDA = 0x1;

	private static final int EXCLUDA = 0x2;

	private static final int NOVA = ~(LIDA | EXCLUDA);

	private String remetente;

	private String assunto;

	private String arquivoudio;

	private Data dataEnvio;

	private int estado;

	/**
	 * Cria uma mensagem de udio de correio eletrnico
	 * 
	 * @param remetente
	 *            O remetente da mensagem
	 * @param assunto
	 *            O assunto da mensagem
	 * @param arquivoudio
	 *            O arquivo contendo o udio da mensagem
	 */
	public MensagemAudio(String remetente, String assunto, String arquivoudio) {
		this.remetente = remetente;
		this.assunto = assunto;
		this.arquivoudio = arquivoudio;
		dataEnvio = new Data();
		estado = NOVA;
	}

	/**
	 * Recupera o remetente da mensagem
	 * 
	 * @return O remetente da mensagem
	 */
	public String getRemetente() {
		return remetente;
	}

	/**
	 * Recupera o assunto da mensagem
	 * 
	 * @return O assunto da mensagem
	 */
	public String getAssunto() {
		return assunto;
	}

	/**
	 * Recupera o arquivo de udio da mensagem.
	 * 
	 * @return O arquivo de udio da mensagem.
	 */
	public String getarquivoudio() {
		return arquivoudio;
	}

	/**
	 * Recupera a data de envio da mensagem
	 * 
	 * @return A data de envio da mensagem
	 */
	public Data getDataEnvio() {
		return dataEnvio;
	}

	/**
	 * Informa se a mensagem foi lida ou no
	 * 
	 * @return true se a mensagem foi lida, false caso contrrio
	 */
	public boolean isLida() {
		return (estado & LIDA) == LIDA;
	}

	/**
	 * Informa se a mensagem foi excluda ou no
	 * 
	 * @return true se a mensagem foi excluda, false caso contrrio
	 */
	public boolean isExcluda() {
		return (estado & EXCLUDA) == EXCLUDA;
	}

	/**
	 * Marcar a mensagem como excluda. A excluso deve ser feita pela coleo
	 * que armazena as mensagens. Um exemplo de tal coleo  CaixaPostal.
	 */
	public void excluir() {
		estado |= EXCLUDA;
	}

	/**
	 * Marcar a mensagem como no excluda.
	 */
	public void marcarNoExcluda() {
		estado &= ~EXCLUDA;
	}

	/**
	 * Marcar a mensagem como no lida.
	 */
	public void marcarNoLida() {
		estado &= ~LIDA;
	}

	/**
	 * Exibir a mensagem. O arquivo de udio  tocado. Aps este mtodo, a
	 * mensagem  considerada "lida".
	 */
	public void exibir() {
		try {
			URL u = new URL("file", "localhost", arquivoudio);
			AudioClip clip = Applet.newAudioClip(u);
			System.out
					.println("Se tiver multimidia no computador, o clip deve estar tocando");
			clip.play();
		} catch (Exception e) {
			System.out.println("Nao pode abrir Audio Clip: " + arquivoudio);
		}
		estado |= LIDA;
	}

	/**
	 * Testa a igualdade de um objeto com esta mensagem.
	 * 
	 * @param objeto
	 *            O objeto a comparar com esta mensagem.
	 * @return true se o objeto for igual a esta mensagem, false caso contrrio.
	 */
	public boolean equals(Object objeto) {
		if (!(objeto instanceof MensagemAudio)) {
			return false;
		}
		MensagemAudio outra = (MensagemAudio) objeto;
		return super.equals(objeto)
				&& getarquivoudio().equals(outra.getarquivoudio());
	}

	/**
	 * Fornea uma representao da mensagem como String
	 * 
	 * @return A representao da mensagem como String.
	 */
	public String toString() {
		return "Remetente: " + remetente + ", Data: "
				+ dataEnvio.DDMMAAAAHHMM() + ", Assunto: " + assunto
				+ ", Arquivo de udio: " + arquivoudio;
	}
}
