import java.util.*;

public class MinhaListaEncadeada implements MinhaLista {
	private Nodo cabealho = new Nodo(null, null, null);

	private int contadorDeElementos = 0;

	public MinhaListaEncadeada() {
		cabealho.prximo = cabealho.anterior = cabealho;
	}

	public int size() {
		return contadorDeElementos;
	}

	public boolean add(Object elemento) {
		adicionarAntes(elemento, cabealho);
		return true;
	}

	private Nodo adicionarAntes(Object elemento, Nodo nodo) {
		Nodo novoNodo = new Nodo(elemento, nodo, nodo.anterior);
		novoNodo.anterior.prximo = novoNodo;
		novoNodo.prximo.anterior = novoNodo;
		contadorDeElementos++;
		return novoNodo;
	}

	public void add(int index, Object elemento) {
		adicionarAntes(elemento, (index == contadorDeElementos ? cabealho
				: nodoNoIndex(index)));
	}

	private Nodo nodoNoIndex(int index) {
		if (index < 0 || index >= contadorDeElementos) {
			throw new IndexOutOfBoundsException("Index: " + index
					+ ", Tamanho: " + contadorDeElementos);
		}
		Nodo nodo = cabealho;
		if (index < contadorDeElementos / 2) {
			for (int i = 0; i <= index; i++) {
				nodo = nodo.prximo;
			}
		} else {
			for (int i = contadorDeElementos; i > index; i--) {
				nodo = nodo.anterior;
			}
		}
		return nodo;
	}

	public Object get(int index) {
		return nodoNoIndex(index).elemento;
	}

	public Object remove(int index) {
		Nodo nodo = nodoNoIndex(index);
		remove(nodo);
		return nodo.elemento;
	}

	public boolean remove(Object elemento) {
		if (elemento == null) {
			for (Nodo nodo = cabealho.prximo; nodo != cabealho; nodo = nodo.prximo) {
				if (nodo.elemento == null) {
					remove(nodo);
					return true;
				}
			}
		} else {
			for (Nodo nodo = cabealho.prximo; nodo != cabealho; nodo = nodo.prximo) {
				if (elemento.equals(nodo.elemento)) {
					remove(nodo);
					return true;
				}
			}
		}
		return false;
	}

	private void remove(Nodo nodo) {
		if (nodo == cabealho) {
			throw new NoSuchElementException();
		}
		nodo.anterior.prximo = nodo.prximo;
		nodo.prximo.anterior = nodo.anterior;
		contadorDeElementos--;
	}

	public void clear() {
		cabealho.prximo = cabealho.anterior = cabealho;
		contadorDeElementos = 0;
	}

	public boolean contains(Object elemento) {
		return indexOf(elemento) >= 0;
	}

	public int indexOf(Object elemento) {
		int index = 0;
		if (elemento == null) {
			for (Nodo nodo = cabealho.prximo; nodo != cabealho; nodo = nodo.prximo) {
				if (nodo.elemento == null) {
					return index;
				}
				index++;
			}
		} else {
			for (Nodo nodo = cabealho.prximo; nodo != cabealho; nodo = nodo.prximo) {
				if (elemento.equals(nodo.elemento)) {
					return index;
				}
				index++;
			}
		}
		return -1;
	}

	public Iterator iterator() {
		return new ListItr();
	}

	public ListIterator listIterator() {
		return new ListItr();
	}

	private class ListItr implements ListIterator {
		private Nodo ltimoRetornado = cabealho; // para saber o que remover
													// no remove()

		private Nodo prximo;

		private int prximoIndex;

		ListItr() {
			prximo = cabealho;
		}

		public boolean hasNext() {
			return prximoIndex != contadorDeElementos;
		}

		public Object next() {
			if (prximoIndex == contadorDeElementos) {
				throw new NoSuchElementException();
			}
			ltimoRetornado = prximo;
			prximo = prximo.prximo;
			prximoIndex++;
			return ltimoRetornado.elemento;
		}

		public boolean hasPrevious() {
			return prximoIndex != 0;
		}

		public Object previous() {
			if (prximoIndex == 0) {
				throw new NoSuchElementException();
			}
			ltimoRetornado = prximo = prximo.anterior;
			prximoIndex--;
			return ltimoRetornado.elemento;
		}

		public int nextIndex() {
			return prximoIndex;
		}

		public int previousIndex() {
			return prximoIndex - 1;
		}

		public void remove() {
			// chamada de remove de outra classe (LinkedList)
			MinhaListaEncadeada.this.remove(ltimoRetornado);
			if (prximo == ltimoRetornado) {
				prximo = ltimoRetornado.prximo;
			} else {
				prximoIndex--;
			}
			ltimoRetornado = cabealho;
		}

		public void set(Object elemento) {
			if (ltimoRetornado == cabealho) {
				throw new IllegalStateException();
			}
			ltimoRetornado.elemento = elemento;
		}

		public void add(Object elemento) {
			ltimoRetornado = cabealho;
			adicionarAntes(elemento, prximo);
			prximoIndex++;
		}
	}

	private static class Nodo {
		Object elemento;

		Nodo prximo;

		Nodo anterior;

		Nodo(Object elemento, Nodo prximo, Nodo anterior) {
			this.elemento = elemento;
			this.prximo = prximo;
			this.anterior = anterior;
		}
	}
}
