/*
 * Desenvolvido para a disciplina Programacao 1
 * Curso de Bacharelado em Cincia da Computao
 * Departamento de Sistemas e Computao
 * Universidade Federal da Paraba
 *
 * Copyright (C) 1999 Universidade Federal da Paraba.
 * No redistribuir sem permisso.
 */
package p1.aplic.banco;

import p1.aplic.geral.*;

/**
 * Classe de conta caixa.
 * A agencia tem uma conta caixa que  uma conta com vrias restries.
 * A conta caixa representa o caixa da agncia. Como todos os movimentos
 * feitos a contas bancrias devem envolver <i>duas</i> contas (pelos
 * princpios da contabilidade), a conta caixa  usada durante os depsitos
 * e saques feitos pelos clientes com suas contas normais.
 * <P>Por exemplo, um depsito envolve uma transferncia de valor <i>para</i>
 * uma conta. De onde veio este dinheiro? Da conta caixa. Em outras palavras,
 * a conta caixa  debitada e a conta alvo  creditada.
 * <p>Da mesma forma, um saque envolve um dbito de uma conta normal e um crdito
 * na conta caixa.
 * <p>A conta caixa tem um comportamento diferente das demais contas.
 * Por exemplo: ela pode ter saldo negativo  vontade (resultados de depsitos)
 * e certas outras operaes lanam excees.
 *
 * @author   Jacques Philippe Sauv, jacques@dsc.ufpb.br
 * @version 1.1
 * <br>
 * Copyright (C) 1999 Universidade Federal da Paraba.
 */
public class ContaCaixa extends Conta {
	// construtores
    /** Constroi uma conta caixa */
	public ContaCaixa() {
		// usa uma pessoa fisica ficticia
		super(new Pessoa("caixa"), 0);
	}

	/**
	 * Transfere um valor do caixa para outra conta (depsito).
	 * @param destino A conta destino.
	 * @param valor O valor a transferir.
	 * @param descr A descrio da transao.
	 * @return O sucesso ou no da operao.
	 */
	public boolean transferir(Conta destino, double valor, String descr) {
		this.receber(-valor);
		destino.receber(valor);
		logarTransacao(this, destino, new Real(valor), descr);
		return true; // sempre pode debitar o caixa: nunca d erro
	}

	/**
	 * No deixa efetuar depsito no caixa. Para que no haja depsito feito
	 * diretamente na conta "caixa", fazemos override de um depsito normal
	 * para que este no seja permitido para a conta caixa.
	 * @param valor O valor que se quer depositar.
	 * @return O sucesso ou no da operao (sempre false).
	 */
	public boolean depositar(double valor) {
		return false; // nao se deve fazer "depsitos" diretos no caixa
	}

	/**
	 * No deixa efetuar saques no caixa. Para que no haja saque feito
	 * diretamente na conta "caixa", fazemos override de um saque normal
	 * para que este no seja permitido para a conta caixa.
	 * @param valor O valor que se quer sacar.
	 * @return O sucesso ou no da operao (sempre d false).
	 */
	public boolean sacar(double valor) {
		return false; // nao se deve fazer "saques" no caixa
	}

	/**
	 * No permite o fechamento da conta
	 * @throws NaoPodeFecharContaException Porque no  permitido fechar o caixa.
	 */
	public void fechar() throws NaoPodeFecharContaException  {
		throw new NaoPodeFecharContaException(this, "Nao pode fechar o caixa");
	}

	/**
	 * Transforma os dados da conta em um String.
	 * @return O string com os dados da conta.
	 */
   	public String toString() {
		return "ContaCaixa saldo " + getSaldoMonetrio();
	}
}
