/*
 * Desenvolvido para a disciplina Programacao 1
 * Curso de Bacharelado em Cincia da Computao
 * Departamento de Sistemas e Computao
 * Universidade Federal da Paraba
 *
 * Copyright (C) 1999 Universidade Federal da Paraba.
 * No redistribuir sem permisso.
 */
package p1.aplic.correio;

import p1.aplic.geral.*;
import java.io.*;

/**
 * Interface para manipular uma mensagem de correio eletronico.
 * Uma mensagem contm um remetente, um assunto uma data de envio e algum contedo.
 * O contedo depende do tipo exato de mensagem (textual, udio).
 * Uma mensagem pode ser exibida (lida) e marcada para excluso.
 *
 * @author   Jacques Philippe Sauv, jacques@dsc.ufpb.br
 * @version 1.1
 * <br>
 * Copyright (C) 1999 Universidade Federal da Paraba.
 */

public interface Mensagem  {
    /**
     * Recupera o remetente da mensagem
     * @return O remetente da mensagem
     */
    public String getRemetente();

    /**
     * Recupera o assunto da mensagem
     * @return O assunto da mensagem
     */
    public String getAssunto();

    /**
     * Recupera a data de envio da mensagem
     * @return A data de envio da mensagem
     */
    public Data getDataEnvio();

    /**
     * Informa se a mensagem foi lida ou no
     * @return true se a mensagem foi lida, false caso contrrio
     */
    public boolean isLida();

    /**
     * Informa se a mensagem foi excluda ou no
     * @return true se a mensagem foi excluda, false caso contrrio
     */
    public boolean isExcluda();

    /**
     * Marcar a mensagem como excluda.
     * A excluso deve ser feita pela coleo que armazena as mensagens.
     * Um exemplo de tal coleo  CaixaPostal.
     */
    public void excluir();

    /**
     * Marcar a mensagem como no excluda.
     */
    public void marcarNoExcluda();

    /**
     * Marcar a mensagem como no lida.
     */
    public void marcarNoLida();

    /**
     * Testa a igualdade de um objeto com esta mensagem.
     * @param objeto O objeto a comparar com esta mensagem.
     * @return true se o objeto for igual a esta mensagem, false caso contrrio.
     */
    public boolean equals(Object objeto);

    /**
     * Exibir a mensagem. Isso poder imprimir algo na sada
     * ou provocar outras sadas relacionadas com a leitura da mensagem.
     * Aps este mtodo, a mensagem  considerada "lida".
     */
    public void exibir();

    /**
     * Fornea uma representao da mensagem como String
     * @return A representao da mensagem como String.
     */
    public String toString();
}