/*
 * Desenvolvido para a disciplina Programacao 1
 * Curso de Bacharelado em Cincia da Computao
 * Departamento de Sistemas e Computao
 * Universidade Federal da Paraba
 *
 * Copyright (C) 1999 Universidade Federal da Paraba.
 * No redistribuir sem permisso.
 */
package p1.aplic.correio;

import p1.aplic.geral.*;
import java.io.*;
import java.net.*;
import java.applet.*;

/**
 * Classe que representa uma mensagem de udio de correio eletronico.
 *
 * @author   Jacques Philippe Sauv, jacques@dsc.ufpb.br
 * @version 1.1
 * <br>
 * Copyright (C) 1999 Universidade Federal da Paraba.
 */

public class MensagemAudio extends MensagemAbstrata implements Serializable {
  static final long serialVersionUID = 4756562040979715146L;
  protected String  arquivoudio;

  /**
   * Cria uma mensagem de udio de correio eletrnico
   * @param remetente O remetente da mensagem
   * @param assunto O assunto da mensagem
   * @param arquivoudio O arquivo contendo o udio da mensagem
   */
  public MensagemAudio(String remetente, String assunto, String arquivoudio) {
    super(remetente, assunto);
    this.arquivoudio = arquivoudio;
  }

  /**
   * Recupera o arquivo de udio da mensagem.
   * @return O arquivo de udio da mensagem.
   */
  public String getarquivoudio() {
    return arquivoudio;
  }

  /**
   * Exibir a mensagem. O arquivo de udio  tocado.
   * Aps este mtodo, a mensagem  considerada "lida".
   */
  public void exibir() {
    try {
      URL u = new URL("file", "localhost", arquivoudio);
      AudioClip clip = Applet.newAudioClip(u);
      System.out.println("Se tiver multimidia no computador, o clip deve estar tocando");
      clip.play();
    } catch(Exception e) {
      System.out.println("Nao pode abrir Audio Clip: " + arquivoudio);
    }
    estado |= LIDA;
  }   

  /**
   * Testa a igualdade de um objeto com esta mensagem.
   * @param objeto O objeto a comparar com esta mensagem.
   * @return true se o objeto for igual a esta mensagem, false caso contrrio.
   */
  public boolean equals(Object objeto) {
    if(! (objeto instanceof MensagemAudio)) {
      return false;
    }
    MensagemAudio outra = (MensagemAudio)objeto;
    return super.equals(objeto) &&
           getarquivoudio().equals(outra.getarquivoudio());
  }

  /**
   * Fornea uma representao da mensagem como String
   * @return A representao da mensagem como String.
   */
  public String toString() {
    return "Remetente: " + remetente +
           ", Data: " + dataEnvio.DDMMAAAAHHMM() +
           ", Assunto: " + assunto +
           ", Arquivo de udio: " + arquivoudio;
  }
}
